#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>

#include "ranktable.h"
#include "getline.h"

#define TEST(r, s) \
    assert(table_find_by_rank(t, (r))); \
    assert(!strcmp(table_find_by_rank(t, (r)), (s))); \
    assert(table_rank(t, (s)) == (r));
#define TEST_NOT_FOUND(s) \
    assert(table_rank(t, (s)) == TABLE_NOT_FOUND);
#define INSERT(s) (t = table_insert(t, (s)))

#define ITERATIONS (100000)

/* test for ranktable.c */
/* reads lines from stdin and prints their rank among those previously read */
int
main(int argc, char **argv)
{
    Table t;
    char buf[512];
    int i;

    /* bad case #2 from compression testing */
    t = EMPTY_TABLE;
    INSERT("");
    INSERT("a");
    INSERT("A");
    INSERT("Aachen");
    TEST(0, "");
    TEST(1, "A");
    TEST(2, "Aachen");
    TEST(3, "a");

    /* possible bad case detected in compression testing */
    t = EMPTY_TABLE;

    TEST_NOT_FOUND("");
    TEST_NOT_FOUND("a");

    t = table_insert(t, "");

    TEST(0, "");
    TEST_NOT_FOUND("a");

    t = table_insert(t, "a");

    TEST(0, "");
    TEST(1, "a");

    table_destroy(t);

    /* basic sanity check */
    assert(table_size(EMPTY_TABLE) == 0);
    assert(table_rank(EMPTY_TABLE, "abc") == TABLE_NOT_FOUND);
    assert(table_find_by_rank(EMPTY_TABLE, 0) == 0);
    t = table_insert(EMPTY_TABLE, "abc");
    assert(table_size(t) == 1);
    TEST(0, "abc");
    TEST_NOT_FOUND("def");
    t = table_insert(t, "def");
    assert(table_size(t) == 2);
    TEST(0, "abc");
    TEST(1, "def");
    TEST_NOT_FOUND("aaa");
    t = table_insert(t, "aaa");
    assert(table_size(t) == 3);
    TEST(0, "aaa");
    TEST(1, "abc");
    TEST(2, "def");
    TEST_NOT_FOUND("aax");
    t = table_insert(t, "");
    assert(table_size(t) == 4);
    TEST(0, "");
    TEST(1, "aaa");
    TEST(2, "abc");
    TEST(3, "def");
    TEST_NOT_FOUND("aax");

    table_destroy(t);

    /* stress test */
    t = EMPTY_TABLE;

    for(i = 0; i < ITERATIONS; i++) {
        sprintf(buf, "%08d", i);
        TEST_NOT_FOUND(buf);
        t = table_insert(t, buf);
        TEST(i, buf);
    }

    for(i = 0; i < ITERATIONS; i+=17) {
        sprintf(buf, "%08d", i);
        TEST(i, buf);
    }

    TEST_NOT_FOUND("");
    t = table_insert(t, "");
    TEST(0, "");

    for(i = 0; i < ITERATIONS; i+=17) {
        sprintf(buf, "%08d", i);
        TEST(i+1, buf);
    }



    table_destroy(t);

    return 0;
}
